
/* error.c - print help and error messages to stderr */

#include "main.h"

/* print help option to stderr */

void	opt_printf (char *msg)
{
	fprintf (stderr, "    %s\n", msg);
}

/* print program usage */

void	print_usage (void)
{
	fprintf (stderr, "scoreproc v1.0.0");
	fprintf (stderr, " - written by Istvan Varga, Jul 2001\n\n");
	fprintf (stderr, "usage: %s [options...]\n\n", argv[0]);
	fprintf (stderr, "options:\n");
	opt_printf ("-h, -help, --help");
	opt_printf ("    Print this help message.");
	opt_printf ("-i, -infile <input file name>");
	opt_printf ("    Sorted score file (output of scsort utility) for");
	opt_printf ("    input. Default is stdin.");
	opt_printf ("-o, -outfile <output file name>");
	opt_printf ("    Output score file. Default is stdout.");
	opt_printf ("-c, -cmdfile <program file name>");
	opt_printf ("    File containing score processing commands (details");
	opt_printf ("    about this file can be found in the README).");
	opt_printf ("    Default: no program file.");
	fprintf (stderr, "\n");
}

/* print error message */

void	err_msg (char *msg, int err_arg)
{
	if (msg != NULL) {
		fprintf (stderr, "*** %s", argv[0]);
		fprintf (stderr, ": %s", msg);
		if (err_arg > 0) fprintf (stderr, ": %s", argv[err_arg]);
		fprintf (stderr, "\n");
	}
	exit (-1);
}

/* print error message and character */

void	err_msg_c (char *msg, int x)
{
	if (msg != NULL) {
		fprintf (stderr, "*** %s: ", argv[0]);
		if ((cmd_line_num) && (cmd_file_name != NULL))
			fprintf (stderr, "in %s, line %d: ",
					 cmd_file_name, cmd_line_num);
		fprintf (stderr, "%s: %c\n", msg, x);
	}
	exit (-1);
}

/* print error message and integer value */

void	err_msg_i (char *msg, int x)
{
	if (msg != NULL) {
		fprintf (stderr, "*** %s: ", argv[0]);
		if ((cmd_line_num) && (cmd_file_name != NULL))
			fprintf (stderr, "in %s, line %d: ",
					 cmd_file_name, cmd_line_num);
		fprintf (stderr, "%s: %d\n", msg, x);
	}
	exit (-1);
}

/* print error message and floating point value */

void	err_msg_d (char *msg, double x)
{
	if (msg != NULL) {
		fprintf (stderr, "*** %s: ", argv[0]);
		if ((cmd_line_num) && (cmd_file_name != NULL))
			fprintf (stderr, "in %s, line %d: ",
					 cmd_file_name, cmd_line_num);
		fprintf (stderr, "%s: %f\n", msg, x);
	}
	exit (-1);
}

/* print error message and string */

void	err_msg_s (char *msg, char *x)
{
	if (msg != NULL) {
		fprintf (stderr, "*** %s: ", argv[0]);
		if ((cmd_line_num) && (cmd_file_name != NULL))
			fprintf (stderr, "in %s, line %d: ",
					 cmd_file_name, cmd_line_num);
		fprintf (stderr, "%s", msg);
		if (x != NULL) fprintf (stderr, ": %s", x);
		fprintf (stderr, "\n");
	}
	exit (-1);
}

/* print usage and error message */

void	err_usage (char *msg, int err_arg)
{
	print_usage ();
	err_msg (msg, err_arg);
}

