
/* scoreproc - score processing utility	*/
/* written by Istvan Varga, Jul 2001	*/

#ifndef SCOREPROC_MAIN_H
#define SCOREPROC_MAIN_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <limits.h>
#include <ctype.h>
#include <math.h>
#ifdef USE_GETTIMEOFDAY
#include <unistd.h>
#include <sys/time.h>
#else
#include <time.h>
#endif

typedef struct {			/* score event structure	*/
	int	nr_pfields;		/* number of p-fields		*/
	double	time_ofs;		/* time offset in beats		*/
	double	end_ofs;		/* time offset for note-off	*/
	double	*p_list;		/* list of p-fields		*/
} score_event;

typedef struct {			/* print format for p-fields	*/
	int	fwidth;			/* field width (in characters)	*/
	int	precision;		/* precision			*/
	double	l_limit;		/* lower limit			*/
	double	u_limit;		/* upper limit			*/
} p_format;

typedef struct {			/* event mask			*/
	int	m_mode;			/* masking mode (0, 1, or 2)	*/
	int	p_num;			/* p-field number		*/
	double	l_limit;		/* lower limit			*/
	double	u_limit;		/* upper limit			*/
	double	loop;			/* loop				*/
} event_mask;

/* global variables */

extern	char		**argv;		/* command line arguments	*/
extern	FILE		*in_file;	/* score file			*/
extern	FILE		*cmd_file;	/* program file			*/
extern	FILE		*out_file;	/* output file			*/
extern	char		*in_file_name;	/* score file name		*/
extern	char		*cmd_file_name;	/* program file name		*/
extern	char		*out_file_name;	/* output file name		*/
extern	int		nr_events;	/* number of events		*/
extern	score_event	*event_list;	/* list of score events		*/
extern	p_format	p_params[];	/* score printing parameters	*/
extern	int		cmd_line_num;	/* line number in program file	*/
extern	event_mask	e_masks[];	/* event masks			*/
extern	unsigned long	r_seed;		/* random seed			*/

#define PI_D	3.14159265358979323846	/* PI				*/
#define TWOPI_D	(2.0 * PI_D)		/* 2 * PI			*/
#define P_MAX	4096			/* max. number of p-fields	*/
#define N_MASKS	4			/* number of event masks	*/

/* -------- functions from score_read.c -------- */

/* read all notes from score */

extern	void	read_score (void);

/* free memory used by event list */

extern	void	free_list (void);

/* -------- functions from score_write.c -------- */

/* set default printing format */

extern	void	init_p_params (void);

/* print all score events */

extern	void	print_score (void);

/* -------- functions from error.c -------- */

/* print help option to stderr */

extern	void	opt_printf (char*);

/* print program usage */

extern	void	print_usage (void);

/* print error message */

extern	void	err_msg (char*, int);

/* print error message and character */

extern	void	err_msg_c (char*, int);

/* print error message and integer value */

extern	void	err_msg_i (char*, int);

/* print error message and floating point value */

extern	void	err_msg_d (char*, double);

/* print error message and string */

extern	void	err_msg_s (char*, char*);

/* print usage and error message */

extern	void	err_usage (char*, int);

/* -------- functions from read_file.c -------- */

/* Read a line from text file. */

extern	int	get_line (FILE*, char***);

/* Read a statement with parameters from text file. */

extern	int	get_statement (FILE*, char**, double**);

/* Check if a double value is in the specified range. */

extern	void	check_value (double, double, double, char*);

/* Convert a double value to int, and check if it is in the specified range. */

extern	int	check_int (double, int, int, char*);

/* -------- functions from cmd_main.c -------- */

/* read program file, and execute score processing statements */

extern	void	cmd_main (void);

/* Get p-field from event list. */

int	read_pfield (double*, int, int);

/* Store p-field in event list. */

int	store_pfield (double, int, int);

/* Return the number of next event, or -1 if none was found. */

int	next_event (int);

#endif		/* SCOREPROC_MAIN_H */

