
/* lib.c - misc. functions */

#include "main.h"

/* calculate average tempo	*/

double	calc_tempo (void)
{
	int	i;
	double	tot_time;

	tot_time = 0.0;
	for (i = 0; i < nr_pts; i++) {
		tot_time += (1.0 / tempo[i][0]);
		tot_time += (1.0 / tempo[i][1]);
	}
	tot_time = 2.0 * (double) nr_pts / tot_time;

	return tot_time;
}

/* normalize tempo array	*/

void	normalize_tempo (void)
{
	double	atmp;
	int	i;

	atmp = 0.0;			/* remove DC offset */
	for (i = 0; i < nr_pts; i++) atmp += (tempo[i][0] + tempo[i][1]);
	atmp = (atmp / (2.0 * (double) nr_pts)) - 1.0;
	for (i = 0; i < nr_pts; i++) {
		tempo[i][0] -= atmp;
		tempo[i][1] -= atmp;
		if (tempo[i][0] < 1.0e-6) tempo[i][0] = 1.0e-6;
		if (tempo[i][1] < 1.0e-6) tempo[i][1] = 1.0e-6;
	}
	atmp = calc_tempo ();		/* normalize average tempo */
	for (i = 0; i < nr_pts; i++) {
		tempo[i][0] *= (avg_tempo / atmp);
		tempo[i][0] = (tempo[i][0] < 1.0e-2 ? 1.0e-2 : tempo[i][0]);
		tempo[i][0] = (tempo[i][0] > 1.0e+4 ? 1.0e+4 : tempo[i][0]);
		tempo[i][1] *= (avg_tempo / atmp);
		tempo[i][1] = (tempo[i][1] < 1.0e-2 ? 1.0e-2 : tempo[i][1]);
		tempo[i][1] = (tempo[i][1] > 1.0e+4 ? 1.0e+4 : tempo[i][1]);
	}
}

/* print tempo values to stdout	*/

void	print_tempo (void)
{
	int	i, k = 0, list_size;
	double	t_avg;

	/* create tempo list */

	list_size = 0;
	if (!(ISEQUAL (start_beat, 0.0))) {
		t_lst[0][0] = 0.0; t_lst[0][1] = avg_tempo; list_size++;
	}
	i = 0; while (i < nr_pts) {
		t_lst[list_size][0] = start_beat + (double) i * beat_time;
		t_lst[list_size][1] = tempo[i][0]; list_size++;
		t_lst[list_size][0] = start_beat + (double) (i + 1) * beat_time;
		t_lst[list_size][1] = tempo[i][1]; list_size++;
		i++;
	}
	t_lst[list_size][0] = start_beat + (double) nr_pts * beat_time;
	t_lst[list_size][1] = avg_tempo; list_size++;

	/* optimize tempo list */

	while ((list_size > 1) && ISEQUAL (t_lst[list_size - 2][1],
					   t_lst[list_size - 1][1]))
		list_size--;

	i = list_size - 1; while (--i >= 0) {
		if (ISEQUAL (t_lst[i][0], t_lst[i + 1][0])
		    && ISEQUAL (t_lst[i][1], t_lst[i + 1][1])) {
			for (k = (i + 1); k < list_size; k++) {
				t_lst[k - 1][0] = t_lst[k][0];
				t_lst[k - 1][1] = t_lst[k][1];
			}
			list_size--;
		}
	}

	i = list_size - 2; while (--i >= 0) {
		t_avg = (1.0 / t_lst[i + 2][1]) - (1.0 / t_lst[i][1]);
		t_avg *= t_lst[i + 1][0] - t_lst[i][0];
		t_avg /= t_lst[i + 2][0] - t_lst[i][0];
		t_avg += (1.0 / t_lst[i][1]);
		t_avg = 1.0 / t_avg;
		if (ISEQUAL (t_avg, t_lst[i + 1][1])) {
			for (k = (i + 2); k < list_size; k++) {
				t_lst[k - 1][0] = t_lst[k][0];
				t_lst[k - 1][1] = t_lst[k][1];
			}
			list_size--;
		}
	}

	/* print tempo values */

	printf ("t");

	for (i = 0; i < list_size; i++) {
		printf ("\t%10.4f\t%10.4f\n", t_lst[i][0], t_lst[i][1]);
	}
}

/* clear tempo data */

void	clear_tempo (void)
{
	int	i;

	for (i = 0; i < MAX_PTS; i++) tempo[i][0] = tempo[i][1] = 1.0;
}

/* update beat_time and nr_pts */

void	update_nr_pts (void)
{
	beat_time = exp (log (2.0) * (double) b_res);
	nr_pts = (int) (0.9999 + (double) nbeats / beat_time);
}

