
/* opts.c - functions implementing command line options */

#include "main.h"

/* print help */

void	opt_help (void)
{
	print_usage ();
	exit (0);
}

/* set average tempo */

void	opt_avg_tempo (void)
{
	nr_args--; argptr++;
	if (nr_args < 1) err_usage ("missing argument", -1);
	avg_tempo = (double) atof (argv[argptr]);
	if ((avg_tempo < 1.0e-2) || (avg_tempo > 1.0e+4))
		err_msg ("invalid tempo value", argptr);
	nr_args--; argptr++;
}

/* set time offset */

void	opt_start_beat (void)
{
	nr_args--; argptr++;
	if (nr_args < 1) err_usage ("missing argument", -1);
	start_beat = (double) atof (argv[argptr]);
	if ((start_beat < 0.0) || (start_beat > 1.0e+4))
		err_msg ("invalid start_beat", argptr);
	nr_args--; argptr++;
}

/* set number of beats */

void	opt_nbeats (void)
{
	nr_args--; argptr++;
	if (nr_args < 1) err_usage ("missing argument", -1);
	nbeats = (int) atoi (argv[argptr]);
	if ((nbeats < 0) || (nbeats > 16384))
		err_msg ("invalid number of beats", argptr);
	update_nr_pts ();
	clear_tempo ();
	nr_args--; argptr++;
}

/* time resolution */

void	opt_b_res (void)
{
	nr_args--; argptr++;
	if (nr_args < 1) err_usage ("missing argument", -1);
	b_res = (int) atoi (argv[argptr]);
	if ((b_res < -8) || (b_res > 8))
		err_msg ("invalid time resolution", argptr);
	update_nr_pts ();
	clear_tempo ();
	nr_args--; argptr++;
}

/* sine modulation */

void	opt_sine (void)
{
	int	i;
	double	arg1, arg2, arg3, arg4, phs0, phs1, min, max;

	nr_args--; argptr++;
	if (nr_args < 4) err_usage ("missing argument", -1);
	arg1 = (double) atof (argv[argptr]);			/* depth */
	if ((arg1 < -1.0) || (arg1 > 1.0))
		err_msg ("invalid modulation depth", argptr);
	arg2 = (double) atof (argv[++argptr]);			/* loop time */
	if ((arg2 < 0.001) || (arg2 > 1.0e+4))
		err_msg ("invalid loop time", argptr);
	arg3 = (double) atof (argv[++argptr]);			/* start phs */
	if ((arg3 < 0.0) || (arg3 > 1.0))
		err_msg ("invalid phase value", argptr);
	arg4 = (double) atof (argv[++argptr]);			/* end phase */
	if ((arg4 < 0.0) || (arg4 > 1.0))
		err_msg ("invalid phase value", argptr);

	arg3 *= 2.0 * PI_D;
	arg4 *= 2.0 * PI_D;
	min = 1.0; max = -1.0;
	if ((arg3 < (0.5 * PI_D)) && (arg4 > (0.5 * PI_D))) max = 1.0;
	if ((arg3 > (0.5 * PI_D)) && (arg4 < (0.5 * PI_D))) max = 1.0;
	if ((arg3 < (1.5 * PI_D)) && (arg4 > (1.5 * PI_D))) min = -1.0;
	if ((arg3 > (1.5 * PI_D)) && (arg4 < (1.5 * PI_D))) min = -1.0;
	if (sin (arg3) > max) max = sin (arg3);
	if (sin (arg4) > max) max = sin (arg4);
	if (sin (arg3) < min) min = sin (arg3);
	if (sin (arg4) < min) min = sin (arg4);
	arg1 *= 2.0 / (fabs (max - min) > 1.0e-6 ? (max - min) : 1.0);
	i = -1; while ((++i < nr_pts) && (i < MAX_PTS)) {
		phs0 = fmod ((double) i * beat_time, arg2) / arg2;
		phs0 = arg3 + (arg4 - arg3) * phs0;
		phs1 = (double) i + 0.99999999;
		phs1 = fmod (phs1 * beat_time, arg2) / arg2;
		phs1 = arg3 + (arg4 - arg3) * phs1;
		tempo[i][0] += arg1 * sin (phs0);
		tempo[i][1] += arg1 * sin (phs1);
	}

	nr_args -= 4; argptr++;
}

/* triangle / saw modulation */

void	opt_trsw (void)
{
	int	i;
	double	arg1, arg2, arg3, arg4, phs0, phs1;

	nr_args--; argptr++;
	if (nr_args < 4) err_usage ("missing argument", -1);
	arg1 = (double) atof (argv[argptr]);			/* depth */
	if ((arg1 < -1.0) || (arg1 > 1.0))
		err_msg ("invalid modulation depth", argptr);
	arg2 = (double) atof (argv[++argptr]);			/* loop time */
	if ((arg2 < 0.001) || (arg2 > 1.0e+4))
		err_msg ("invalid loop time", argptr);
	arg3 = (double) atof (argv[++argptr]);			/* attack */
	if ((arg3 < 0.0) || (arg3 > 1.0))
		err_msg ("invalid attack value", argptr);
	arg4 = (double) atof (argv[++argptr]);			/* phase */
	if ((arg4 < 0.0) || (arg4 > 1.0))
		err_msg ("invalid phase value", argptr);

	i = -1; while ((++i < nr_pts) && (i < MAX_PTS)) {
		phs0 = fmod ((double) i * beat_time, arg2) / arg2;
		phs0 = fmod (phs0 + arg4, 1.0);
		phs1 = (double) i + 0.99999999;
		phs1 = fmod (phs1 * beat_time, arg2) / arg2;
		phs1 = fmod (phs1 + arg4, 1.0);
		if ((arg3 == 1.0) || ((arg3 != 0.0) && (phs0 < arg3))) {
			phs0 = 2.0 * (phs0 / arg3) - 1.0;
		} else {
			phs0 = 1.0 - 2.0 * ((phs0 - arg3) / (1.0 - arg3));
		}
		if ((arg3 == 1.0) || ((arg3 != 0.0) && (phs1 < arg3))) {
			phs1 = 2.0 * (phs1 / arg3) - 1.0;
		} else {
			phs1 = 1.0 - 2.0 * ((phs1 - arg3) / (1.0 - arg3));
		}
		tempo[i][0] += arg1 * phs0;
		tempo[i][1] += arg1 * phs1;
	}

	nr_args -= 4; argptr++;
}

